import QtQuick 2.0
import QtGraphicalEffects 1.0
import Sailfish.Silica 1.0
import Nemo.Configuration 1.0
import org.SfietKonstantin.patchmanager 2.0
import "./presets.js" as PresetData

//import "./components"

Page {
    id: page
    allowedOrientations: Orientation.All
    property bool pmTranslationLoaded: PatchManagerTranslator ? PatchManagerTranslator.installTranslator("patch-icon-filter") : false

    ConfigurationGroup {
        id: conf
        path: "/apps/patchmanager/patch-icon-filter"
        property double brightness: -0.2
        property double contrast: 0.1
        property double desaturation: 0.3
        property int last_preset
        property bool customColor: false
        property color customColorColor: "gray"
    }

    property bool custom_enabled: false
    property alias custom_color: conf.customColor


    SilicaFlickable {
        anchors.fill: parent
        contentHeight: column.height

        VerticalScrollDecorator{}

        PresetModel {
            id: presetModel
            Component.onCompleted: init()
        }

        ContextMenu {
            id: presetMenu
            Repeater {
                model: presetModel
                MenuItem { text: display_name}
                Component.onCompleted: console.debug("repeated: " + count + "from " + model)
            }
        }
        Column {
            id: column
            width: parent.width
            spacing: Theme.paddingMedium

            PageHeader {
                title: qsTr("Filter Settings")
            }

            SectionHeader {
                text: qsTr("Description")
            }

            Label {
                text: qsTr("Adjust the values for the effects to be applied to the icons.  " +
                    "Once the patch is applied, changes here will take effect immediately."
                )
                wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                width: parent.width - 2*x
                x: Theme.horizontalPageMargin
                font.pixelSize: Theme.fontSizeSmall
            }

            Item {
                width: parent.width - 2*x
                height: iconRowPlain.height
                x: Theme.horizontalPageMargin
                Row {
                    id: iconRowPlain
                    spacing: Theme.paddingLarge
                    anchors.horizontalCenter: parent.horizontalCenter
                    height: Theme.iconSizeLarge
                    Image { source: "image://theme/icon-launcher-gallery"}
                    Image { source: "image://theme/icon-launcher-email"}
                    Image { source: "image://theme/icon-launcher-messaging"}
                    Image { source: "image://theme/icon-launcher-calendar"}
                }
            }
            Item {
                width: parent.width - 2*x
                height: iconRow.height
                x: Theme.horizontalPageMargin
                anchors.horizontalCenter: parent.horizontalCenter
                Row {
                    id: iconRow
                    spacing: Theme.paddingLarge
                    anchors.horizontalCenter: parent.horizontalCenter
                    height: Theme.iconSizeLarge
                    Image { source: "image://theme/icon-launcher-gallery"}
                    Image { source: "image://theme/icon-launcher-email"}
                    Image { source: "image://theme/icon-launcher-messaging"}
                    Image { source: "image://theme/icon-launcher-calendar"}
                }
                BrightnessContrast {
                    id: iconRowEff1
                    source: iconRow
                    anchors.centerIn: source
                    height: source.height
                    width: source.width

                    // hide to daisy-chain with next effect
                    visible: false
                    cached: visible

                    brightness: conf.brightness
                    contrast: conf.contrast
                }

                Desaturate {
                    id: iconRowEff2
                    source: iconRowEff1
                    anchors.centerIn: source
                    height: source.height
                    width: source.width
                    // hide to daisy-chain with next effect
                    visible:  !iconRowEffColor.visible
                    cached: visible
                    desaturation: conf.desaturation
                }
                ColorOverlay {
                    id: iconRowEffColor
                    source: iconRow
                    anchors.centerIn: source
                    height: source.height
                    width: source.width
                    cached: visible
                    visible: conf.customColor

                    // custom tint
                    color: conf.customColorColor
                }
            }

            SectionHeader {
                text: qsTr("Presets")
            }

            Label {
                text: qsTr("Select predefined sets, or \"%1\" to customize").arg(qsTr("User Defined"))
                wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                width: parent.width - 2*x
                x: Theme.horizontalPageMargin
                font.pixelSize: Theme.fontSizeSmall
            }
            ComboBox {
                id: selector
                anchors.horizontalCenter: parent.horizontalCenter
                label: qsTr("Preset" +":")
                menu: presetMenu

                currentIndex: conf.last_preset ? conf.last_preset : presetModel.default_idx
                Component.onCompleted: {
                    presetModel.init();
                    currentIndex = conf.last_preset ? conf.last_preset : presetModel.default_idx
                }
                onCurrentIndexChanged: {
                    selector.description = presetModel.get(currentIndex).description ? presetModel.get(currentIndex).description : ""
                    if (!currentItem) return;
                    if (presetModel.get(currentIndex).name ==  "custom") {
                        page.custom_enabled = true;
                        page.custom_color = false;
                    } else if (presetModel.get(currentIndex).name ==  "custom_color") {
                        page.custom_color = true;
                        page.custom_enabled = false;
                        conf.setValue("customColor", true);
                        conf.sync();
                    } else {
                        page.custom_enabled = false;
                        page.custom_color = false;
                        conf.brightness   = presetModel.get(currentIndex).brightness;
                        conf.contrast     = presetModel.get(currentIndex).contrast;
                        conf.desaturation = presetModel.get(currentIndex).desaturation;
                        conf.sync();
                    }
                    conf.last_preset  = selector.currentIndex;
                }
            }
            Separator { width: parent.width; horizontalAlignment: Qt.AlignRight; color: Theme.primaryColor }

            SectionHeader {
                opacity: page.custom_enabled ? 1.0 : Theme.opacityLow
                text: qsTr("Brightness") + "/" + qsTr("Contrast")
            }

            Slider {
                enabled:  page.custom_enabled; opacity: enabled? 1.0 : Theme.opacityLow
                Behavior on opacity { FadeAnimation {} }
                width: parent.width
                minimumValue: -0.8
                maximumValue: 0.8
                stepSize: 0.01
                value: conf.brightness
                valueText: value.toPrecision(2)
                handleVisible: true
                label: qsTr("Brightness")
                onReleased:  { conf.setValue("brightness", sliderValue.toPrecision(3)) }
            }

            Slider {
                enabled:  page.custom_enabled; opacity: enabled? 1.0 : Theme.opacityLow
                Behavior on opacity { FadeAnimation {} }
                width: parent.width
                minimumValue: -0.8
                maximumValue: 0.8
                stepSize: 0.01
                value: conf.contrast
                valueText: value.toPrecision(2)
                handleVisible: true
                label: qsTr("Contrast")
                onReleased:  { conf.setValue("contrast", sliderValue.toPrecision(3)) }
            }

            SectionHeader {
                opacity: page.custom_enabled ? 1.0 : Theme.opacityLow
                text: qsTr("Desaturation")
            }

            Slider {
                enabled:  page.custom_enabled; opacity: enabled? 1.0 : Theme.opacityLow
                Behavior on opacity { FadeAnimation {} }
                width: parent.width
                minimumValue: 0.0
                maximumValue: 1.0
                stepSize: 0.01
                value: conf.desaturation
                valueText: value.toPrecision(2)
                handleVisible: true
                label: qsTr("Desaturation")
                onReleased:  { conf.setValue("desaturation", sliderValue.toPrecision(3)) }
            }

            SectionHeader {
                text: qsTr("Color Overlay")
                opacity: page.custom_color ? 1.0 : Theme.opacityLow
            }
            Slider { id: strengthSlider
                enabled:  page.custom_color; opacity: enabled? 1.0 : Theme.opacityLow
                Behavior on opacity { FadeAnimation {} }
                width: parent.width
                minimumValue: 0.0
                maximumValue: 0.75
                stepSize: 0.01
                value: conf.customColorColor.a ? conf.customColorColor.a : 0.2
                valueText: Math.floor((value*100)) + "%"
                handleVisible: true
                label: qsTr("Strength")
                onReleased: { conf.setValue( "customColorColor", Qt.rgba(redSlider.value, greenSlider.value, blueSlider.value, strengthSlider.value)) }
            }
            Rectangle{
                opacity: page.custom_color ? 1.0 : Theme.opacityLow
                anchors.horizontalCenter: parent.horizontalCenter
                height: Theme.paddingLarge * 2
                width: height * 16/9
                color: Qt.rgba(redSlider.value, greenSlider.value, blueSlider.value, 1.0)
                Text {
                    color: Qt.rgba( 0.5 - parent.color.r, 0.5 - parent.color.g,   0.5 - parent.color.b, 1.0)
                    text: parent.color.toString()
                    anchors.centerIn: parent
                    font.pixelSize: Theme.fontSizeSmall
                }
            }
            Slider { id: redSlider
                color: Theme.highlightFromColor("red", Theme.colorScheme)
                enabled:  page.custom_color; opacity: enabled? 1.0 : Theme.opacityLow
                Behavior on opacity { FadeAnimation {} }
                width: parent.width
                minimumValue: 0.0
                maximumValue: 1.0
                stepSize: maximumValue / 255
                value: conf.customColorColor.r
                valueText: Math.floor((value*255))
                handleVisible: true
                label: qsTr("Red Channel")
                onReleased: { conf.setValue( "customColorColor", Qt.rgba(redSlider.value, greenSlider.value, blueSlider.value, strengthSlider.value)) }
            }
            Slider { id: greenSlider
                color: Theme.highlightFromColor("green", Theme.colorScheme)
                enabled:  page.custom_color; opacity: enabled? 1.0 : Theme.opacityLow
                Behavior on opacity { FadeAnimation {} }
                width: parent.width
                minimumValue: 0.0
                maximumValue: 1.0
                stepSize: maximumValue / 255
                value: conf.customColorColor.g
                valueText: Math.floor((value*255))
                handleVisible: true
                label: qsTr("Green Channel")
                onReleased: { conf.setValue( "customColorColor", Qt.rgba(redSlider.value, greenSlider.value, blueSlider.value, strengthSlider.value)) }
            }
            Slider { id: blueSlider
                color: Theme.highlightFromColor("blue", Theme.colorScheme)
                enabled:  page.custom_color; opacity: enabled? 1.0 : Theme.opacityLow
                Behavior on opacity { FadeAnimation {} }
                width: parent.width
                minimumValue: 0.0
                maximumValue: 1.0
                stepSize: maximumValue / 255
                value: conf.customColorColor.b
                valueText: Math.floor((value*255))
                handleVisible: true
                label: qsTr("Blue Channel")
                onReleased: { conf.setValue( "customColorColor", Qt.rgba(redSlider.value, greenSlider.value, blueSlider.value, strengthSlider.value)) }
            }
        }

    }
}

// vim: expandtab ts=4 st=4 sw=4 filetype=javascript
